@@ -1,37 +1,43 @@
+# This file is part of datacube-ows, part of the Open Data Cube project.
+# See https://opendatacube.org for more information.
+#
+# Copyright (c) 2017-2021 OWS Contributors
+# SPDX-License-Identifier: Apache-2.0
 from __future__ import absolute_import
 
 import json
+import logging
+import re
+from collections import OrderedDict
 from datetime import datetime
+from itertools import chain
 
+import datacube
 import numpy
 import numpy.ma
 import xarray
+from datacube.utils import geometry
+from datacube.utils.masking import mask_to_dict
+from pandas import Timestamp
 from rasterio.io import MemoryFile
 from rasterio.warp import Resampling
 from skimage.draw import polygon as skimg_polygon
-from itertools import chain
-import re
-
-import datacube
-from datacube.utils import geometry
-from datacube.utils.masking import mask_to_dict
 
 from datacube_ows.cube_pool import cube
+from datacube_ows.mv_index import MVSelectOpts, mv_search
 from datacube_ows.ogc_exceptions import WMSException
-
+from datacube_ows.ogc_utils import (ConfigException, dataset_center_time,
+                                    solar_date, tz_for_geometry,
+                                    xarray_image_as_png)
 from datacube_ows.ows_configuration import get_config
 from datacube_ows.query_profiler import QueryProfiler
-from datacube_ows.wms_utils import img_coords_to_geopoint, GetMapParameters, \
-    GetFeatureInfoParameters, solar_correct_data
-from datacube_ows.ogc_utils import dataset_center_time, ConfigException, tz_for_geometry, \
-    solar_date, xarray_image_as_png
-from datacube_ows.mv_index import MVSelectOpts, mv_search
 from datacube_ows.utils import log_call
-
-import logging
+from datacube_ows.wms_utils import (GetFeatureInfoParameters, GetMapParameters,
+                                    img_coords_to_geopoint, solar_correct_data)
 
 _LOG = logging.getLogger(__name__)
 
+
 class ProductBandQuery:
     def __init__(self, products, bands, main=False, manual_merge=False, ignore_time=False, fuse_func=None):
         self.products = products
@@ -88,7 +94,6 @@ class ProductBandQuery:
                                    manual_merge=layer.data_manual_merge,
                                    fuse_func=layer.fuse_func,
                                    resource_limited=False)
-
         ]
         for fpb in layer.allflag_productbands:
             if fpb.products_match(layer.product_names):
@@ -126,7 +131,7 @@ class DataStacker:
         if style:
             self._needed_bands = list(style.needed_bands)
         elif bands:
-            self._needed_bands = [ self._product.band_idx.band(b) for b in bands ]
+            self._needed_bands = [self._product.band_idx.band(b) for b in bands]
         else:
             self._needed_bands = list(self._product.band_idx.native_bands.index)
 
@@ -184,7 +189,7 @@ class DataStacker:
             times = None
         else:
             times = self._times
-        results = {}
+        results = []
         for query in queries:
             if query.ignore_time:
                 qry_times = None
@@ -199,10 +204,26 @@ class DataStacker:
                 result = datacube.Datacube.group_datasets(result, self.group_by)
                 if all_time:
                     return result
-                results[query] = result
+                results.append((query, result))
             else:
                 return result
-        return results
+        return OrderedDict(results)
+
+    def create_zero_filled_flag_bands(self, data, pbq):
+        var = "No data vars in result"
+        for var in data.data_vars.variables.keys():
+            break
+        template = getattr(data, var)
+        new_data = numpy.ndarray(template.shape, dtype="uint8")
+        new_data.fill(0)
+        qry_result = template.copy(data=new_data)
+        data_new_bands = {}
+        for band in pbq.bands:
+            data_new_bands[band] = qry_result
+        data = data.assign(data_new_bands)
+        for band in pbq.bands:
+            data[band].attrs["flags_definition"] = pbq.products[0].measurements[band].flags_definition
+        return data
 
     @log_call
     def data(self, datasets_by_query, skip_corrections=False):
@@ -210,6 +231,9 @@ class DataStacker:
         # datasets is an XArray DataArray of datasets grouped by time.
         data = None
         for pbq, datasets in datasets_by_query.items():
+            if data is not None and len(data.time) == 0:
+                # No data, so no need for masking data.
+                continue
             measurements = pbq.products[0].lookup_measurements(pbq.bands)
             fuse_func = pbq.fuse_func
             if pbq.manual_merge:
@@ -219,30 +243,37 @@ class DataStacker:
             if data is None:
                 data = qry_result
                 continue
+            if len(data.time) == 0:
+                # No data, so no need for masking data.
+                continue
             if pbq.ignore_time:
                 # regularise time dimension:
-                if len(qry_result.time) != 1:
+                if len(qry_result.time) > 1:
                     raise WMSException("Cannot ignore time on PQ (flag) bands from a time-aware product")
-                if len(qry_result.time) == len(data.time):
+                elif len(qry_result.time) == len(data.time):
                     qry_result["time"] = data.time
                 else:
-                    data_new_bands = {}
-                    for band in pbq.bands:
-                        band_data = qry_result[band]
-                        timeless_band_data = band_data.sel(time=qry_result.time.values[0])
-                        band_time_slices = []
-                        for dt in data.time.values:
-                            band_time_slices.append(timeless_band_data)
-                        timed_band_data = xarray.concat(band_time_slices, data.time)
-                        data_new_bands[band] = timed_band_data
-
+                    if len(qry_result.time) == 0:
+                        data = self.create_zero_filled_flag_bands(data, pbq)
+                        continue
+                    else:
+                        data_new_bands = {}
+                        for band in pbq.bands:
+                            band_data = qry_result[band]
+                            timeless_band_data = band_data.sel(time=qry_result.time.values[0])
+                            band_time_slices = []
+                            for dt in data.time.values:
+                                band_time_slices.append(timeless_band_data)
+                            timed_band_data = xarray.concat(band_time_slices, data.time)
+                            data_new_bands[band] = timed_band_data
                     data = data.assign(data_new_bands)
                     continue
-            for band in pbq.bands:
-                data = data.assign({
-                    band: qry_result[band]
-                    for band in pbq.bands
-                })
+            elif len(qry_result.time) == 0:
+                # Time-aware mask product has no data, but main product does.
+                data = self.create_zero_filled_flag_bands(data, pbq)
+                continue
+            qry_result.coords["time"] = data.coords["time"]
+            data = xarray.combine_by_coords([data, qry_result], join="exact")
 
         return data
 
@@ -322,6 +353,26 @@ def bbox_to_geom(bbox, crs):
     return datacube.utils.geometry.box(bbox.left, bbox.bottom, bbox.right, bbox.top, crs)
 
 
+def user_date_sorter(odc_dates, geom, user_dates):
+    tz = tz_for_geometry(geom)
+    result = []
+    for npdt in odc_dates:
+        ts = Timestamp(npdt).tz_localize("UTC")
+        solar_day = solar_date(ts, tz)
+        for idx, date in enumerate(user_dates):
+            if date == solar_day:
+                result.append(idx)
+                break
+    npresult = numpy.array(result, dtype="uint8")
+    xrresult = xarray.DataArray(
+        npresult,
+        coords={"time": odc_dates},
+        dims=["time"],
+        name="user_date_sorter"
+    )
+    return xrresult
+
+
 @log_call
 def get_map(args):
     # pylint: disable=too-many-nested-blocks, too-many-branches, too-many-statements, too-many-locals
@@ -426,6 +477,13 @@ def get_map(args):
                 body = _write_empty(params.geobox)
             else:
                 qprof["write_action"] = "Write Data"
+                if mdh and  mdh.preserve_user_date_order:
+                    sorter = user_date_sorter(data.time.values,
+                                          params.geobox.geographic_extent,
+                                          params.times)
+                    data = data.sortby(sorter)
+                    extent_mask = extent_mask.sortby(sorter)
+
                 body = _write_png(data, params.style, extent_mask, params.geobox, qprof)
 
     if params.ows_stats:
@@ -472,6 +530,7 @@ def _write_empty(geobox):
             pass
         return memfile.read()
 
+
 def get_coordlist(geo, layer_name):
     if geo.type == 'Polygon':
         coordinates_list = [geo.json["coordinates"]]
@@ -567,7 +626,7 @@ def get_s3_browser_uris(datasets, pt=None, s3url="", s3bucket=""):
 @log_call
 def _make_band_dict(prod_cfg, pixel_dataset):
     band_dict = {}
-    for k,v in pixel_dataset.data_vars.items():
+    for k, v in pixel_dataset.data_vars.items():
         band_val = pixel_dataset[k].item()
         flag_def = pixel_dataset[k].attrs.get("flags_definition")
         if flag_def:
@@ -623,6 +682,7 @@ def geobox_is_point(geobox):
     # TODO: Not 100% sure why this function is needed.
     return geobox.height == 1 and geobox.width == 1
 
+
 @log_call
 def feature_info(args):
     # pylint: disable=too-many-nested-blocks, too-many-branches, too-many-statements, too-many-locals
@@ -693,7 +753,10 @@ def feature_info(args):
                         ds = dss.sel(time=dt).values.tolist()[0]
                         break
                 if params.product.multi_product:
-                    date_info["source_product"] = "%s (%s)" % (ds.type.name, ds.metadata_doc["platform"]["code"])
+                    if "platform" in ds.metadata_doc:
+                        date_info["source_product"] = "%s (%s)" % (ds.type.name, ds.metadata_doc["platform"]["code"])
+                    else:
+                        date_info["source_product"] = ds.type.name
 
                 # Extract data pixel
                 pixel_ds = td.isel(**isel_kwargs)
@@ -798,6 +861,7 @@ def feature_info(args):
     }
     return json_response(result, cfg)
 
+
 def json_response(result, cfg=None):
     if not cfg:
         cfg = get_config()
