@@ -1,4 +1,10 @@
 # pylint: skip-file
+# This file is part of datacube-ows, part of the Open Data Cube project.
+# See https://opendatacube.org for more information.
+#
+# Copyright (c) 2017-2021 OWS Contributors
+# SPDX-License-Identifier: Apache-2.0
+
 
 # Example configuration file for datacube_ows.
 #
@@ -112,38 +118,38 @@ landsat8_bands = {
     #
     "red": [],
     "green": [],
-    "blue": [ "near_blue" ],
-    "nir": [ "near_infrared" ],
-    "swir1": [ "shortwave_infrared_1", "near_shortwave_infrared" ],
-    "swir2": [ "shortwave_infrared_2", "far_shortwave_infrared" ],
-    "coastal_aerosol": [ "far_blue" ],
+    "blue": ["near_blue"],
+    "nir": ["near_infrared"],
+    "swir1": ["shortwave_infrared_1", "near_shortwave_infrared"],
+    "swir2": ["shortwave_infrared_2", "far_shortwave_infrared"],
+    "coastal_aerosol": ["far_blue"],
 
     # N.B. Include pixel quality bands if they are in the main data product.
 }
 
-sentinel2_bands= {
-    "nbar_coastal_aerosol": [ 'nbar_far_blue' ],
+sentinel2_bands = {
+    "nbar_coastal_aerosol": ['nbar_far_blue'],
     "nbar_blue": [],
     "nbar_green": [],
     "nbar_red": [],
     "nbar_red_edge_1": [],
     "nbar_red_edge_2": [],
     "nbar_red_edge_3": [],
-    "nbar_nir_1":  [ "nbar_near_infrared_1" ],
-    "nbar_nir_2":  [ "nbar_near_infrared_2" ],
-    "nbar_swir_2": [ "nbar_shortwave_infrared_2" ],
-    "nbar_swir_3": [ "nbar_shortwave_infrared_3" ],
-    "nbart_coastal_aerosol": [ 'coastal_aerosol', 'nbart_far_blue', 'far_blue'],
-    "nbart_blue": [ 'blue' ],
-    "nbart_green": [ 'green' ],
-    "nbart_red": [ 'red' ],
-    "nbart_red_edge_1": [ 'red_edge_1' ],
-    "nbart_red_edge_2": [ 'red_edge_2' ],
-    "nbart_red_edge_3": [ 'red_edge_3' ],
-    "nbart_nir_1":  [ "nir_1", "nbart_near_infrared_1" ],
-    "nbart_nir_2":  [ "nir_2", "nbart_near_infrared_2" ],
-    "nbart_swir_2": [ "swir_2", "nbart_shortwave_infrared_2" ],
-    "nbart_swir_3": [ "swir_3", "nbart_shortwave_infrared_3" ],
+    "nbar_nir_1": ["nbar_near_infrared_1"],
+    "nbar_nir_2": ["nbar_near_infrared_2"],
+    "nbar_swir_2": ["nbar_shortwave_infrared_2"],
+    "nbar_swir_3": ["nbar_shortwave_infrared_3"],
+    "nbart_coastal_aerosol": ['coastal_aerosol', 'nbart_far_blue', 'far_blue'],
+    "nbart_blue": ['blue'],
+    "nbart_green": ['green'],
+    "nbart_red": ['red'],
+    "nbart_red_edge_1": ['red_edge_1'],
+    "nbart_red_edge_2": ['red_edge_2'],
+    "nbart_red_edge_3": ['red_edge_3'],
+    "nbart_nir_1": ["nir_1", "nbart_near_infrared_1"],
+    "nbart_nir_2": ["nir_2", "nbart_near_infrared_2"],
+    "nbart_swir_2": ["swir_2", "nbart_shortwave_infrared_2"],
+    "nbart_swir_3": ["swir_3", "nbart_shortwave_infrared_3"],
 
     # N.B. Include pixel quality bands if they are in the main data product.
     "quality": [],
@@ -368,7 +374,7 @@ style_mineral_content = {
     },
     # If ANY components include a function callback, the bands that need to be passed to the callback
     # MUST be declared in a "additional_bands" item:
-    "additional_bands": [ "red", "blue", "nir", "swir1", "swir2" ]
+    "additional_bands": ["red", "blue", "nir", "swir1", "swir2"]
 
     #
     # The style scale_range can be omitted if all components have a component-specific scale_range defined or
@@ -768,8 +774,8 @@ style_ndvi_delta = {
         #  [ left, bottom, width, height ], as passed to Matplotlib Figure.add_axes function.
         # All values as fractions of the width and height.  (i.e. between 0.0 and 1.0)
         # The default is:
-        "strip_location": [ 0.05, 0.5, 0.9, 0.15 ]
-    } ,
+        "strip_location": [0.05, 0.5, 0.9, 0.15]
+    },
     # Define behaviour(s) for multi-date requests. If not declared, style only supports single-date requests.
     "multi_date": [
         # A multi-date handler.  Different handlers can be declared for different numbers of dates in a request.
@@ -777,6 +783,10 @@ style_ndvi_delta = {
             # The count range for which this handler is to be used - a tuple of two ints, the smallest and
             # largest date counts for which this handler will be used.  Required.
             "allowed_count_range": [2, 2],
+            # Re-sort data returned from the ODC so the order of date coordinates in the "time" dimension
+            # matches the date order supplied by the user in the WMS request.
+            # Optional. Defaults to False (date coordinates in the "time" dimension are always sorted chronologically)
+            "preserve_user_date_order": True,
             # A function, expressed in the standard format as described elsewhere in this example file.
             # The function is assumed to take one arguments, an xarray Dataset.
             # The function returns an xarray Dataset with a single band, which is the input to the
@@ -825,7 +835,7 @@ style_deform = {
     # Legend section is optional for non-linear colour-ramped styles.
     # If not supplied, a legend for the style will be automatically generated from the colour ramp.
     "legend": {
-        ## Only use positive part of range.
+        # Only use positive part of range.
         # tick labels will be created for values that
         # are modulo 0 by this value
         "ticks_every": "10",
@@ -1241,7 +1251,7 @@ ows_cfg = {
         "response_headers": {
             "Access-Control-Allow-Origin": "*",  # CORS header (strongly recommended)
         },
-        ## Which web service(s) should be implemented by this instance
+        # Which web service(s) should be implemented by this instance
         # Optional, defaults: wms,wmts: True, wcs: False
         "services": {
             "wms": True,
@@ -1253,7 +1263,7 @@ ows_cfg = {
         # Service URL.
         # A list of fully qualified URLs that the service can return
         # in the GetCapabilities documents based on the requesting url
-        "allowed_urls": [ "http://localhost/odc_ows",
+        "allowed_urls": ["http://localhost/odc_ows",
                           "https://localhost/odc_ows",
                           "https://alternateurl.domain.org/odc_ows",
                           "http://127.0.0.1:5000/"],
@@ -1288,6 +1298,29 @@ ows_cfg = {
             "fax": "+61 2 1234 6789",
             "email": "test@example.com",
         },
+        # Attribution.
+        #
+        # This provides a way to identify the source of the data used in a WMS layer or layers.
+        # This entire section is optional.  If provided, it is taken as the
+        # default attribution for any layer that does not override it.
+        "attribution": {
+            # Attribution must contain at least one of ("title", "url" and "logo")
+            # A human readable title for the attribution - e.g. the name of the attributed organisation
+            "title": "Acme Satellites",
+            # The associated - e.g. URL for the attributed organisation
+            "url": "http://www.acme.com/satellites",
+            # Logo image - e.g. for the attributed organisation
+            "logo": {
+                # Image width in pixels (optional)
+                "width": 370,
+                # Image height in pixels (optional)
+                "height": 73,
+                # URL for the logo image. (required if logo specified)
+                "url": "https://www.acme.com/satellites/images/acme-370x73.png",
+                # Image MIME type for the logo - should match type referenced in the logo url (required if logo specified.)
+                "format": "image/png",
+            }
+        },
         # If fees are charged for the use of the service, these can be described here in free text.
         # If blank or not included, defaults to "none".
         "fees": "",
@@ -1308,6 +1341,11 @@ ows_cfg = {
                 "geographic": True,
                 "vertical_coord_first": True
             },
+            "EPSG:3111": {  # VicGrid94 for delwp.vic.gov.au
+               "geographic": False,
+                "horizontal_coord": "x",
+                "vertical_coord": "y",
+            },
             "EPSG:3577": {  # GDA-94, internal representation
                 "geographic": False,
                 "horizontal_coord": "x",
@@ -1330,27 +1368,7 @@ ows_cfg = {
         # Optional, defaults to 256x256
         "max_width": 512,
         "max_height": 512,
-        # Attribution. This provides a way to identify the source of the data used in a layer or layers.
-        # This entire section is optional.  If provided, it is taken as the
-        # default attribution for any layer that does not override it.
-        "attribution": {
-            # Attribution must contain at least one of ("title", "url" and "logo")
-            # A human readable title for the attribution - e.g. the name of the attributed organisation
-            "title": "Acme Satellites",
-            # The associated - e.g. URL for the attributed organisation
-            "url": "http://www.acme.com/satellites",
-            # Logo image - e.g. for the attributed organisation
-            "logo": {
-                # Image width in pixels (optional)
-                "width": 370,
-                # Image height in pixels (optional)
-                "height": 73,
-                # URL for the logo image. (required if logo specified)
-                "url": "https://www.acme.com/satellites/images/acme-370x73.png",
-                # Image MIME type for the logo - should match type referenced in the logo url (required if logo specified.)
-                "format": "image/png",
-            }
-        },
+
         # These define the AuthorityURLs.
         # They represent the authorities that define the "Identifiers" defined layer by layer below.
         # The spec allows AuthorityURLs to be defined anywhere on the Layer heirarchy, but datacube_ows treats them
@@ -1522,8 +1540,11 @@ ows_cfg = {
                     # If "dynamic" is False (the default) the the ranges for the product are cached in memory.
                     # Dynamic products slow down the generation of the GetCapabilities document - use sparingly.
                     "dynamic": False,
-                    # The resolution of the time access.  Optional. Allowed values are: "raw" (the default - daily),
-                    # "month" (for monthly summary datasets) or "year" (for annual summary datasets)
+                    # The resolution of the time access.  Optional. Allowed values are:
+                    # * "raw" (the default - data with timestamps to be according to the local solar day)
+                    # * "day" (daily data with date-resolution time stamps)
+                    # * "month" (for monthly summary datasets)
+                    # * "year" (for annual summary datasets)
                     "time_resolution": "raw",
                     "flags": {
                         # Data may include flags that mark which pixels have missing or poor-quality data,
@@ -1620,13 +1641,13 @@ ows_cfg = {
                         # This is the number of CRS units (e.g. degrees, metres) per pixel in the horizontal
                         # and vertical # directions for the native resolution.
                         # E.g. for EPSG:3577; (25.0,25.0) for Landsat-8 and (10.0,10.0 for Sentinel-2)
-                        "native_resolution": [ 25.0, 25.0 ],
+                        "native_resolution": [25.0, 25.0],
                         # The default bands for a WCS request.
                         # 1. Must be provided if WCS is activated.
                         # 2. Must contain at least one band.
                         # 3. All bands must exist in the band index.
                         # 4. Bands may be referred to by either native name or alias
-                        "default_bands": [ "red", "green", "blue" ],
+                        "default_bands": ["red", "green", "blue"],
                         # The native format advertised for the coverage.
                         # Must be one of the formats defined
                         # in the global wcs formats section.
@@ -1752,7 +1773,7 @@ ows_cfg = {
                         # Bands to always fetch from the Datacube, even if it is not used by the active style.
                         # Useful for when a particular band is always needed for the extent_mask_func, as
                         # is the case here.
-                        "always_fetch_bands": [ "quality" ],
+                        "always_fetch_bands": ["quality"],
                         "fuse_func": None,
                         "manual_merge": True,
                         # Apply corrections for solar angle, for "Level 1" products.
@@ -1761,8 +1782,8 @@ ows_cfg = {
                     },
                     "wcs": {
                         "native_crs": "EPSG:3857",
-                        "native_resolution": [ 25.0, 25.0 ],
-                        "default_bands": [ "red", "green", "blue" ],
+                        "native_resolution": [25.0, 25.0],
+                        "default_bands": ["red", "green", "blue"],
                     },
                     "styling": {
                         "default_style": "simple_rgb",
@@ -1785,7 +1806,7 @@ ows_cfg = {
                     "abstract": "Water Observations from Space - Summary",
                     "name": "wofs_summary",
                     "product_name": "wofs_summary",
-                    "bands": { "frequency": [] },
+                    "bands": {"frequency": []},
                     "resource_limits": standard_resource_limits,
                     "flags": None,
                     "image_processing": {
@@ -1794,8 +1815,8 @@ ows_cfg = {
                     },
                     "wcs": {
                         "native_crs": "EPSG:3857",
-                        "native_resolution": [ 25.0, 25.0 ],
-                        "default_bands": [ "frequency" ],
+                        "native_resolution": [25.0, 25.0],
+                        "default_bands": ["frequency"],
                     },
                     "styling": {
                         "styles": [
@@ -1836,15 +1857,15 @@ ows_cfg = {
                     },
                     "image_processing": {
                         "extent_mask_func": "datacube_ows.ogc_utils.mask_by_val",
-                        "always_fetch_bands": [ ],
+                        "always_fetch_bands": [],
                         "fuse_func": None,
                         "manual_merge": False,
                         "apply_solar_corrections": False,
                     },
                     "wcs": {
                         "native_crs": "EPSG:3577",
-                        "native_resolution": [ 10.0, 10.0 ],
-                        "default_bands": [ "red", "green", "blue" ],
+                        "native_resolution": [10.0, 10.0],
+                        "default_bands": ["red", "green", "blue"],
                     },
                     "identifiers": {
                         "auth": "s2_nrt_multi",
@@ -1865,7 +1886,7 @@ ows_cfg = {
                     },
                     "styling": {
                         "default_style": "simple_rgb",
-                        "styles": [ style_rgb ],
+                        "styles": [style_rgb],
                     }
                 } ##### End of sentinel2_nrt multi-product definition
             ],
@@ -1877,20 +1898,20 @@ ows_cfg = {
             "title": "Mangrove Canopy Cover",
             "abstract": "Mangrove Canopy Cover - example of bitflag value-mapped style.",
             "product_names": "mangrove_cover",
-            "bands": { "canopy_cover_class": [], "extent": [] },
+            "bands": {"canopy_cover_class": [], "extent": []},
             "resource_limits": standard_resource_limits,
             "flags": None,
             "image_processing": {
                 "extent_mask_func": "datacube_ows.ogc_utils.mask_by_extent_flag",
-                "always_fetch_bands": [ "extent" ],
+                "always_fetch_bands": ["extent"],
                 "fuse_func": None,
                 "manual_merge": False,
                 "apply_solar_corrections": False,
             },
             "wcs": {
                 "native_crs": "EPSG:3577",
-                "native_resolution": [ 25.0, 25.0 ],
-                "default_bands": [ "canopy_cover_class" ],
+                "native_resolution": [25.0, 25.0],
+                "default_bands": ["canopy_cover_class"],
             },
             "identifiers": {
                 "auth": "mangrove_canopy_cover",
@@ -1911,12 +1932,8 @@ ows_cfg = {
             },
             "styling": {
                 "default_style": "mangrove",
-                "styles": [ style_mangrove ],
+                "styles": [style_mangrove],
             }
         } ##### End of mangrove_cover definition
     ]  ##### End of "layers" list.
 } #### End of example configuration object
-
-
-
-
