@@ -1,3 +1,9 @@
+# This file is part of datacube-ows, part of the Open Data Cube project.
+# See https://opendatacube.org for more information.
+#
+# Copyright (c) 2017-2021 OWS Contributors
+# SPDX-License-Identifier: Apache-2.0
+
 #
 #  Note this is NOT the configuration file!
 #
@@ -5,28 +11,33 @@
 #
 #  Refer to the documentation for information on how to configure datacube_ows.
 #
-
-import os
-import math
-from importlib import import_module
+import datetime
 import json
-
+import logging
+import math
+import os
 from collections.abc import Mapping
+from importlib import import_module
 
+from babel.messages.catalog import Catalog
+from babel.messages.pofile import read_po
+from datacube.utils import geometry
 from ows import Version
 from slugify import slugify
 
-from datacube.utils import geometry
-from datacube_ows.config_utils import cfg_expand, load_json_obj, import_python_obj, OWSConfigEntry, \
-    OWSEntryNotFound, OWSExtensibleConfigEntry, OWSFlagBand, FlagProductBands
+from datacube_ows.config_utils import (FlagProductBands, OWSConfigEntry,
+                                       OWSEntryNotFound,
+                                       OWSExtensibleConfigEntry, OWSFlagBand,
+                                       OWSMetadataConfig, cfg_expand,
+                                       get_file_loc, import_python_obj,
+                                       load_json_obj)
 from datacube_ows.cube_pool import cube, get_cube, release_cube
+from datacube_ows.ogc_utils import (ConfigException, FunctionWrapper,
+                                    create_geobox, day_summary_date_range,
+                                    local_solar_date_range, month_date_range,
+                                    year_date_range)
 from datacube_ows.styles import StyleDef
-from datacube_ows.ogc_utils import ConfigException, FunctionWrapper, month_date_range, local_solar_date_range, \
-    year_date_range, create_geobox
 from datacube_ows.tile_matrix_sets import TileMatrixSet
-
-import logging
-
 from datacube_ows.utils import group_by_statistical
 
 _LOG = logging.getLogger(__name__)
@@ -42,8 +53,7 @@ def read_config(path=None):
         from datacube_ows.ows_cfg import ows_cfg as cfg
     elif "/" in cfg_env or cfg_env.endswith(".json"):
         cfg = load_json_obj(cfg_env)
-        abs_path =  os.path.abspath(cfg_env)
-        cwd = os.path.dirname(abs_path)
+        cwd = get_file_loc(cfg_env)
     elif "." in cfg_env:
         cfg = import_python_obj(cfg_env)
     elif cfg_env.startswith("{"):
@@ -98,9 +108,9 @@ class BandIndex(OWSConfigEntry):
         super().make_ready(dc, *args, **kwargs)
 
     def band(self, name_alias):
-        if name_alias not in self._idx:
-            raise ConfigException(f"Unknown band name/alias: {name_alias} in layer {self.product.name}")
-        return self._idx[name_alias]
+        if name_alias in self._idx:
+            return self._idx[name_alias]
+        raise ConfigException(f"Unknown band name/alias: {name_alias} in layer {self.product.name}")
 
     def band_label(self, name_alias):
         name = self.band(name_alias)
@@ -121,9 +131,9 @@ class BandIndex(OWSConfigEntry):
 
 
 class AttributionCfg(OWSConfigEntry):
-    def __init__(self, cfg):
+    def __init__(self, cfg, owner):
         super().__init__(cfg)
-        self.title = cfg.get("title")
+        self.owner = owner
         self.url = cfg.get("url")
         logo = cfg.get("logo")
         if not self.title and not self.url and not logo:
@@ -141,12 +151,16 @@ class AttributionCfg(OWSConfigEntry):
             if not self.logo_url or not self.logo_fmt:
                 raise ConfigException("url and format must both be specified in an attribution logo.")
 
+    @property
+    def title(self):
+        return self.owner.attribution_title
+
     @classmethod
-    def parse(cls, cfg):
+    def parse(cls, cfg, owner):
         if not cfg:
             return None
         else:
-            return cls(cfg)
+            return cls(cfg, owner)
 
 
 class SuppURL(OWSConfigEntry):
@@ -154,7 +168,7 @@ class SuppURL(OWSConfigEntry):
     def parse_list(cls, cfg):
         if not cfg:
             return []
-        return [ cls(u) for u in cfg ]
+        return [cls(u) for u in cfg]
 
     def __init__(self, cfg):
         super().__init__(cfg)
@@ -163,40 +177,38 @@ class SuppURL(OWSConfigEntry):
 
 
 
-class OWSLayer(OWSConfigEntry):
+class OWSLayer(OWSMetadataConfig):
+    METADATA_KEYWORDS = True
+    METADATA_ATTRIBUTION = True
+
     named = False
-    def __init__(self, cfg, parent_layer=None, **kwargs):
+    def __init__(self, cfg, object_label, parent_layer=None, **kwargs):
         super().__init__(cfg, **kwargs)
+        self.object_label = object_label
         self.global_cfg = kwargs["global_cfg"]
         self.parent_layer = parent_layer
 
-        if "title" not in cfg:
-            raise ConfigException("Layer without title found under parent layer %s" % str(parent_layer))
-        self.title = cfg["title"]
-        if "abstract" in cfg:
-            self.abstract = cfg["abstract"]
-        elif parent_layer:
-            self.abstract = parent_layer.abstract
-        else:
-            raise ConfigException("No abstract supplied for top-level layer %s" % self.title)
-        # Accumulate keywords
-        self.keywords = set()
-        if self.parent_layer:
-            for word in self.parent_layer.keywords:
-                self.keywords.add(word)
-        else:
-            for word in self.global_cfg.keywords:
-                self.keywords.add(word)
-        for word in cfg.get("keywords", []):
-            self.keywords.add(word)
+        self.parse_metadata(cfg)
         # Inherit or override attribution
         if "attribution" in cfg:
-            self.attribution = AttributionCfg.parse(cfg.get("attribution"))
+            self.attribution = AttributionCfg.parse(cfg.get("attribution"), self)
         elif parent_layer:
             self.attribution = self.parent_layer.attribution
         else:
             self.attribution = self.global_cfg.attribution
 
+    def global_config(self):
+        return self.global_cfg
+
+    def can_inherit_from(self):
+        if self.parent_layer:
+            return self.parent_layer
+        else:
+            return self.global_cfg
+
+    def get_obj_label(self):
+        return self.object_label
+
     def layer_count(self):
         return 0
 
@@ -208,25 +220,36 @@ class OWSLayer(OWSConfigEntry):
 
 
 class OWSFolder(OWSLayer):
-    def __init__(self, cfg, global_cfg, parent_layer=None, **kwargs):
-        super().__init__(cfg, parent_layer, global_cfg=global_cfg, **kwargs)
+    def __init__(self, cfg, global_cfg, parent_layer=None, sibling=0, **kwargs):
+        if "label" in cfg:
+            obj_lbl = f"folder.{cfg['label']}"
+        elif parent_layer:
+            obj_lbl = f"{parent_layer.object_label}.{sibling}"
+        else:
+            obj_lbl = f"folder.{sibling}"
+        if obj_lbl in global_cfg.folder_index:
+            raise ConfigException(f"Duplicate folder label: {obj_lbl}")
+        super().__init__(cfg, parent_layer=parent_layer, object_label=obj_lbl, global_cfg=global_cfg, **kwargs)
         self.slug_name = slugify(self.title, separator="_")
         self.unready_layers = []
         self.child_layers = []
         if "layers" not in cfg:
             raise ConfigException("No layers section in folder layer %s" % self.title)
+        child = 0
         for lyr_cfg in cfg["layers"]:
             try:
-                lyr = parse_ows_layer(lyr_cfg, global_cfg, parent_layer=self)
+                lyr = parse_ows_layer(lyr_cfg, global_cfg=global_cfg, parent_layer=self, sibling=child)
                 self.unready_layers.append(lyr)
             except ConfigException as e:
                 _LOG.error("Could not parse layer: %s", str(e))
+            child += 1
+        global_cfg.folder_index[obj_lbl] = self
 
     def unready_layer_count(self):
-        return sum([ l.layer_count() for l in self.unready_layers ])
+        return sum([l.layer_count() for l in self.unready_layers])
 
     def layer_count(self):
-        return sum([ l.layer_count() for l in self.child_layers ])
+        return sum([l.layer_count() for l in self.child_layers])
 
     def make_ready(self, dc, *args, **kwargs):
         still_unready = []
@@ -293,10 +316,12 @@ class CacheControlRules(OWSConfigEntry):
 
 
 TIMERES_RAW = "raw"
+TIMERES_DAY = "day"
 TIMERES_MON = "month"
 TIMERES_YR  = "year"
 
-TIMERES_VALS = [ TIMERES_RAW, TIMERES_MON, TIMERES_YR]
+TIMERES_VALS = [TIMERES_RAW, TIMERES_DAY, TIMERES_MON, TIMERES_YR]
+
 
 class OWSNamedLayer(OWSExtensibleConfigEntry, OWSLayer):
     INDEX_KEYS = ["layer"]
@@ -304,7 +329,7 @@ class OWSNamedLayer(OWSExtensibleConfigEntry, OWSLayer):
 
     def __init__(self, cfg, global_cfg, parent_layer=None, **kwargs):
         name = cfg["name"]
-        super().__init__(cfg, global_cfg=global_cfg, parent_layer=parent_layer,
+        super().__init__(cfg, object_label=f"layer.{name}", global_cfg=global_cfg, parent_layer=parent_layer,
                          keyvals={"layer": name},
                          **kwargs)
         self.name = name
@@ -327,6 +352,11 @@ class OWSNamedLayer(OWSExtensibleConfigEntry, OWSLayer):
         self.declare_unready("product")
         self.declare_unready("definition")
 
+        if global_cfg.user_band_math_extension:
+            self.user_band_math = cfg.get("user_band_math", False)
+        else:
+            self.user_band_math = False
+
         self.time_resolution = cfg.get("time_resolution", TIMERES_RAW)
         if self.time_resolution not in TIMERES_VALS:
             raise ConfigException(
@@ -375,7 +405,6 @@ class OWSNamedLayer(OWSExtensibleConfigEntry, OWSLayer):
         # And finally, add to the global product index.
         self.global_cfg.product_index[self.name] = self
 
-
     # pylint: disable=attribute-defined-outside-init
     def make_ready(self, dc, *args, **kwargs):
         self.products = []
@@ -432,9 +461,9 @@ class OWSNamedLayer(OWSExtensibleConfigEntry, OWSLayer):
     def parse_image_processing(self, cfg):
         emf_cfg = cfg["extent_mask_func"]
         if isinstance(emf_cfg, Mapping) or isinstance(emf_cfg, str):
-            self.extent_mask_func = [ FunctionWrapper(self, emf_cfg) ]
+            self.extent_mask_func = [FunctionWrapper(self, emf_cfg)]
         else:
-            self.extent_mask_func = list([ FunctionWrapper(self, emf) for emf in emf_cfg ])
+            self.extent_mask_func = list([FunctionWrapper(self, emf) for emf in emf_cfg])
         self.raw_afb = cfg.get("always_fetch_bands", [])
         self.declare_unready("always_fetch_bands")
         self.solar_correction = cfg.get("apply_solar_corrections", False)
@@ -451,13 +480,13 @@ class OWSNamedLayer(OWSExtensibleConfigEntry, OWSLayer):
 
     # pylint: disable=attribute-defined-outside-init
     def ready_image_processing(self, dc):
-        self.always_fetch_bands = list([ self.band_idx.band(b) for b in self.raw_afb ])
+        self.always_fetch_bands = list([self.band_idx.band(b) for b in self.raw_afb])
 
     # pylint: disable=attribute-defined-outside-init
     def parse_feature_info(self, cfg):
         self.feature_info_include_utc_dates = cfg.get("include_utc_dates", False)
         custom = cfg.get("include_custom", {})
-        self.feature_info_custom_includes = { k: FunctionWrapper(self, v) for k,v in custom.items() }
+        self.feature_info_custom_includes = {k: FunctionWrapper(self, v) for k, v in custom.items()}
 
     # pylint: disable=attribute-defined-outside-init
     def parse_flags(self, cfg):
@@ -479,7 +508,7 @@ class OWSNamedLayer(OWSExtensibleConfigEntry, OWSLayer):
                 raise ConfigException(f"Product name mismatch in flags section for layer {self.name}: product_names {pns} has multiple distinct low-res product names")
             pq_names_to_lowres_names[pns] = lrpns
         # pylint: disable=dict-values-not-iterating
-        self.allflag_productbands = FlagProductBands.build_list_from_flagbands(self.flag_bands.values())
+        self.allflag_productbands = FlagProductBands.build_list_from_flagbands(self.flag_bands.values(), self)
 
     # pylint: disable=attribute-defined-outside-init
     def parse_urls(self, cfg):
@@ -559,9 +588,10 @@ class OWSNamedLayer(OWSExtensibleConfigEntry, OWSLayer):
             try:
                 native_bounding_box = self.bboxes[self.native_CRS]
             except KeyError:
-                _LOG.warning("Layer: %s No bounding box in ranges for native CRS %s - rerun update_ranges.py",
-                             self.name,
-                             self.native_CRS)
+                if not self.global_cfg.called_from_update_ranges:
+                    _LOG.warning("Layer: %s No bounding box in ranges for native CRS %s - rerun update_ranges.py",
+                                 self.name,
+                                 self.native_CRS)
                 self.hide = True
                 return
             self.origin_x = native_bounding_box["left"]
@@ -676,7 +706,8 @@ class OWSNamedLayer(OWSExtensibleConfigEntry, OWSLayer):
             self.bboxes = self.extract_bboxes()
         # pylint: disable=broad-except
         except Exception as a:
-            _LOG.warning("get_ranges failed for layer %s: %s", self.name, str(a))
+            if not self.global_cfg.called_from_update_ranges:
+                _LOG.warning("get_ranges failed for layer %s: %s", self.name, str(a))
             self.hide = True
             self.bboxes = {}
         finally:
@@ -712,6 +743,10 @@ class OWSNamedLayer(OWSExtensibleConfigEntry, OWSLayer):
     def is_raw_time_res(self):
         return self.time_resolution == TIMERES_RAW
 
+    @property
+    def is_day_time_res(self):
+        return self.time_resolution == TIMERES_DAY
+
     @property
     def is_month_time_res(self):
         return self.time_resolution == TIMERES_MON
@@ -725,12 +760,14 @@ class OWSNamedLayer(OWSExtensibleConfigEntry, OWSLayer):
             return month_date_range(t)
         elif self.is_year_time_res:
             return year_date_range(t)
+        elif self.is_day_time_res:
+            return day_summary_date_range(t)
         else:
             if not geobox:
                 bbox = self.ranges["bboxes"][self.native_CRS]
-                geobox=create_geobox(
+                geobox = create_geobox(
                     self.native_CRS,
-                    bbox["left"], bbox["bottom"], bbox["right"],bbox["top"],
+                    bbox["left"], bbox["bottom"], bbox["right"], bbox["top"],
                     1, 1
                 )
             return local_solar_date_range(geobox, t)
@@ -834,14 +871,15 @@ class OWSMultiProductLayer(OWSNamedLayer):
             "pq_low_res_names": pq_low_res_names,
         }
 
-def parse_ows_layer(cfg, global_cfg, parent_layer=None):
+
+def parse_ows_layer(cfg, global_cfg, parent_layer=None, sibling=0):
     if cfg.get("name", None):
         if cfg.get("multi_product", False):
             return OWSMultiProductLayer(cfg, global_cfg, parent_layer)
         else:
             return OWSProductLayer(cfg, global_cfg, parent_layer)
     else:
-        return OWSFolder(cfg, global_cfg, parent_layer)
+        return OWSFolder(cfg, global_cfg, parent_layer=parent_layer, sibling=sibling)
 
 
 class WCSFormat:
@@ -898,7 +936,51 @@ class WCSFormat:
         return self.renderers[version]
 
 
-class OWSConfig(OWSConfigEntry):
+class ContactInfo(OWSConfigEntry):
+    def __init__(self, cfg, global_cfg):
+        super().__init__(cfg)
+        self.global_cfg = global_cfg
+        self.person = cfg.get("person")
+
+        class Address(OWSConfigEntry):
+            def __init__(self, cfg):
+                super().__init__(cfg)
+                self.type = cfg.get("type")
+                self.address = cfg.get("address")
+                self.city = cfg.get("city")
+                self.state = cfg.get("state")
+                self.postcode = cfg.get("postcode")
+                self.country = cfg.get("country")
+
+            @classmethod
+            def parse(cls, cfg):
+                if not cfg:
+                    return None
+                else:
+                    return cls(cfg)
+
+        self.address = Address.parse(cfg.get("address"))
+        self.telephone = cfg.get("telephone")
+        self.fax = cfg.get("fax")
+        self.email = cfg.get("email")
+
+    @property
+    def organisation(self):
+        return self.global_cfg.contact_org
+
+    @property
+    def position(self):
+        return self.global_cfg.contact_position
+
+    @classmethod
+    def parse(cls, cfg, global_cfg):
+        if cfg:
+            return cls(cfg, global_cfg)
+        else:
+            return None
+
+
+class OWSConfig(OWSMetadataConfig):
     _instance = None
     initialised = False
 
@@ -907,14 +989,23 @@ class OWSConfig(OWSConfigEntry):
             cls._instance = super().__new__(cls)
         return cls._instance
 
-    def __init__(self, refresh=False, cfg=None):
+    METADATA_KEYWORDS = True
+    METADATA_ATTRIBUTIONS = True
+    METADATA_FEES = True
+    METADATA_ACCESS_CONSTRAINTS = True
+    METADATA_CONTACT_INFO = True
+
+    default_abstract = ""
+
+    def __init__(self, refresh=False, cfg=None, ignore_msgfile=False, called_from_update_ranges=False):
+        self.called_from_update_ranges = called_from_update_ranges
         if not self.initialised or refresh:
+            self.msgfile = None
             if not cfg:
                 cfg = read_config()
             super().__init__(cfg)
-            self.initialised = True
             try:
-                self.parse_global(cfg["global"])
+                self.parse_global(cfg["global"], ignore_msgfile)
             except KeyError as e:
                 raise ConfigException(
                     "Missing required config entry in 'global' section: %s" % str(e)
@@ -948,33 +1039,78 @@ class OWSConfig(OWSConfigEntry):
                 raise ConfigException(
                     "Missing required config entry in 'wmts' section (with WCS enabled): %s" % str(e)
                 )
+            self.catalog = None
+            self.initialised = True
 
     #pylint: disable=attribute-defined-outside-init
     def make_ready(self, dc, *args, **kwargs):
+        if self.msg_file_name:
+            try:
+                with open(self.msg_file_name, "rb") as fp:
+                    self.set_msg_src(read_po(fp, locale=self.default_locale, domain=self.message_domain))
+            except FileNotFoundError:
+                _LOG.warning("Message file %s does not exist - using metadata from config file", self.msg_file_name)
+        else:
+            self.set_msg_src(None)
         self.native_product_index = {}
         self.root_layer_folder.make_ready(dc, *args, **kwargs)
         super().make_ready(dc, *args, **kwargs)
 
-    def parse_global(self, cfg):
+    def export_metadata(self):
+        if self.catalog is None:
+            now = datetime.datetime.now()
+            self.catalog = Catalog(locale=self.default_locale,
+                                   domain=self.message_domain,
+                                   header_comment=f"""# Translations for datacube-ows metadata instance:
+#      {self.title}
+#
+# {self.contact_info.organisation} {now.isoformat()} 
+#""",
+                                   project=self.title,
+                                   version=f"{now.isoformat()}",
+                                   copyright_holder=self.contact_info.organisation,
+                                   msgid_bugs_address=self.contact_info.email,
+                                   creation_date=now,
+                                   revision_date=now,
+                                   fuzzy=False)
+            for k, v in self._metadata_registry.items():
+                if self._inheritance_registry[k]:
+                    continue
+                if k in [
+                        "folder.ows_root_hidden.title",
+                        "folder.ows_root_hidden.abstract",
+                        "folder.ows_root_hidden.local_keywords",
+                 ]:
+                    continue
+                self.catalog.add(id=k, string=v, auto_comments=[v])
+        return self.catalog
+
+    def parse_global(self, cfg, ignore_msgfile):
         self._response_headers = cfg.get("response_headers", {})
         self.wms = cfg.get("services", {}).get("wms", True)
         self.wmts = cfg.get("services", {}).get("wmts", True)
         self.wcs = cfg.get("services", {}).get("wcs", False)
         if not self.wms and not self.wmts and not self.wcs:
             raise ConfigException("At least one service must be active.")
-        self.title = cfg["title"]
+        self.locales = cfg.get("supported_languages", ["en"])
+        if len(self.locales) < 1:
+            raise ConfigException("You must support at least one language.")
+        self.default_locale = self.locales[0]
+        self.message_domain = cfg.get("message_domain", "ows_cfg")
+        self.translations_dir = cfg.get("translations_directory")
+        self.internationalised = self.translations_dir and len(self.locales) > 1
+        if self.internationalised:
+            _LOG.info("Internationalisation enabled.")
+        if ignore_msgfile:
+            self.msg_file_name = None
+        else:
+            self.msg_file_name = cfg.get("message_file")
+        self.parse_metadata(cfg)
         self.allowed_urls = cfg["allowed_urls"]
         self.info_url = cfg["info_url"]
-        self.abstract = cfg.get("abstract")
-        self.contact_info = cfg.get("contact_info", {})
-        self.keywords = cfg.get("keywords", [])
-        self.fees = cfg.get("fees")
-        self.access_constraints = cfg.get("access_constraints")
-        # self.use_extent_views = cfg.get("use_extent_views", False)
-        if not self.fees:
-            self.fees = "none"
-        if not self.access_constraints:
-            self.access_constraints = "none"
+        self.contact_info = ContactInfo.parse(cfg.get("contact_info"), self)
+        self.attribution = AttributionCfg.parse(cfg.get("attribution"), self)
+
         def make_gml_name(name):
             if name.startswith("EPSG:"):
                 return f"http://www.opengis.net/def/crs/EPSG/0/{name[5:]}"
@@ -1023,8 +1159,10 @@ class OWSConfig(OWSConfigEntry):
         self.s3_aws_zone = cfg.get("s3_aws_zone", "")
         self.wms_max_width = cfg.get("max_width", 256)
         self.wms_max_height = cfg.get("max_height", 256)
-        self.attribution = AttributionCfg.parse(cfg.get("attribution"))
         self.authorities = cfg.get("authorities", {})
+        self.user_band_math_extension = cfg.get("user_band_math_extension", False)
+        if "attribution" in cfg:
+            _LOG.warning("Attribution entry in top level 'wms' section will be ignored. Attribution should be moved to the 'global' section")
 
     def parse_wcs(self, cfg):
         if self.wcs:
@@ -1070,13 +1208,14 @@ class OWSConfig(OWSConfigEntry):
             self.tile_matrix_sets[identifier] = TileMatrixSet(identifier, tms, self)
 
     def parse_layers(self, cfg):
+        self.folder_index = {}
         self.product_index = {}
         self.declare_unready("native_product_index")
         self.root_layer_folder = OWSFolder({
             "title": "Root Folder (hidden)",
-            "abstract": ".",
+            "label": "ows_root_hidden",
             "layers": cfg
-        }, self, None)
+        }, global_cfg=self, parent_layer=None)
 
     @property
     def layers(self):
@@ -1111,8 +1250,8 @@ class OWSConfig(OWSConfigEntry):
         return hdrs
 
 
-def get_config(refresh=False):
-    cfg = OWSConfig(refresh=refresh)
+def get_config(refresh=False, called_from_update_ranges=False):
+    cfg = OWSConfig(refresh=refresh, called_from_update_ranges=called_from_update_ranges)
     if not cfg.ready:
         with cube() as dc:
             cfg.make_ready(dc)
