@@ -1,19 +1,26 @@
-from collections import defaultdict
-from decimal import Decimal, ROUND_HALF_UP
-from math import isclose
+# This file is part of datacube-ows, part of the Open Data Cube project.
+# See https://opendatacube.org for more information.
+#
+# Copyright (c) 2017-2021 OWS Contributors
+# SPDX-License-Identifier: Apache-2.0
 import logging
+from collections import defaultdict
+from decimal import ROUND_HALF_UP, Decimal
+from math import isclose
 
 import matplotlib
 import numpy
 from colour import Color
 from matplotlib import pyplot as plt
-from matplotlib.colors import to_hex, LinearSegmentedColormap
+from matplotlib.colors import LinearSegmentedColormap, to_hex
 from xarray import Dataset
 
 matplotlib.use('Agg')
 
 from datacube_ows.ogc_utils import ConfigException, FunctionWrapper
 from datacube_ows.styles.base import StyleDefBase
+from datacube_ows.styles.expression import Expression
+
 _LOG = logging.getLogger(__name__)
 
 UNSCALED_DEFAULT_RAMP = [
@@ -60,7 +67,7 @@ def scale_unscaled_ramp(rmin, rmax, unscaled):
         rmax = float(rmax)
     return [
         {
-            "value": (rmax - rmin)*u["value"] + rmin,
+            "value": (rmax - rmin) * u["value"] + rmin,
             "color": u["color"],
             "alpha": u.get("alpha", 1.0)
         } for u in unscaled
@@ -84,7 +91,7 @@ def crack_ramp(ramp):
     return (values, red, green, blue, alpha)
 
 
-def read_mpl_ramp(mpl_ramp : str):
+def read_mpl_ramp(mpl_ramp: str):
     unscaled_cmap = []
     try:
         cmap = plt.get_cmap(mpl_ramp)
@@ -94,17 +101,17 @@ def read_mpl_ramp(mpl_ramp : str):
     rgba_hex = to_hex(cmap(0.0))
     unscaled_cmap.append(
         {
-            "value" : 0.0,
-            "color" : rgba_hex,
-            "alpha" : 1.0
+            "value": 0.0,
+            "color": rgba_hex,
+            "alpha": 1.0
         }
     )
     for val in val_range:
         rgba_hex = to_hex(cmap(val))
         unscaled_cmap.append(
             {
-                "value" : float(val),
-                "color" : rgba_hex
+                "value": float(val),
+                "color": rgba_hex
             }
         )
     return unscaled_cmap
@@ -198,8 +205,8 @@ def legacy_colour_ramp_legend(bytesio, legend_cfg, colour_ramp, map_name,
     # ticks are also normalized between 0 - 1.0
     # so they are position correctly on the colorbar
     def create_cdict_ticks(cfg, ramp):
-        generate = (cfg.get("major_ticks", None) is not None or \
-                    cfg.get("scale_by", None) is not None or \
+        generate = (cfg.get("major_ticks", None) is not None or
+                    cfg.get("scale_by", None) is not None or
                     cfg.get("radix_point", None) is not None)
 
         return from_definition(cfg, ramp, generate)
@@ -382,7 +389,7 @@ class ColorRamp:
 
         self.auto_legend = True
         self.legend_title = cfg.get("title")
-        self.legend_units = cfg.get("units","")
+        self.legend_units = cfg.get("units", "")
         self.legend_decimal_places = cfg.get("decimal_places", 1)
         if self.legend_decimal_places < 0:
             raise ConfigException("decimal_places cannot be negative")
@@ -488,7 +495,7 @@ class ColorRamp:
         self.legend_width = cfg.get("width", 4)
         self.legend_height = cfg.get("height", 1.25)
         self.legend_strip_location = cfg.get("strip_location",
-                                      [ 0.05, 0.5, 0.9, 0.15])
+                                      [0.05, 0.5, 0.9, 0.15])
 
     def get_value(self, data, band):
         return numpy.interp(data, self.values, self.components[band])
@@ -519,26 +526,32 @@ class ColorRamp:
 
 class ColorRampDef(StyleDefBase):
     auto_legend = True
-    def __init__(self, product, style_cfg, stand_alone=False, defer_multi_date=False):
+    def __init__(self, product, style_cfg, stand_alone=False, defer_multi_date=False, user_defined=False):
         super(ColorRampDef, self).__init__(product, style_cfg,
-                           stand_alone=stand_alone, defer_multi_date=defer_multi_date)
+                           stand_alone=stand_alone, defer_multi_date=defer_multi_date, user_defined=user_defined)
         style_cfg = self._raw_cfg
         self.color_ramp = ColorRamp(self, style_cfg)
-
-        for band in style_cfg["needed_bands"]:
-            self.raw_needed_bands.add(band)
-
         self.include_in_feature_info = style_cfg.get("include_in_feature_info", True)
 
         if "index_function" in style_cfg:
             self.index_function = FunctionWrapper(self,
                                                   style_cfg["index_function"],
                                                   stand_alone=self.stand_alone)
+            if not self.stand_alone:
+                for band in style_cfg["needed_bands"]:
+                    self.raw_needed_bands.add(band)
+        elif "index_expression" in style_cfg:
+            self.index_function = Expression(self, style_cfg["index_expression"])
+            for band in self.index_function.needed_bands:
+                self.raw_needed_bands.add(band)
+            if self.stand_alone:
+                self.needed_bands = [self.local_band(b) for b in self.raw_needed_bands]
+                self.flag_bands = []
         else:
             raise ConfigException("Index function is required for index and hybrid styles. Style %s in layer %s" % (
                 self.name,
                 self.product.name
-            ) )
+            ))
         if not defer_multi_date:
             self.parse_multi_date(style_cfg)
 
