@@ -1,29 +1,26 @@
+# This file is part of datacube-ows, part of the Open Data Cube project.
+# See https://opendatacube.org for more information.
+#
+# Copyright (c) 2017-2021 OWS Contributors
+# SPDX-License-Identifier: Apache-2.0
 from __future__ import absolute_import, division, print_function
 
 from flask import request
-
-
-from ows.common.v20.decoders import kvp_decode_get_capabilities
 from ows.common import WGS84BoundingBox
-from ows.gml import Grid, RegularAxis, IrregularAxis, SpatioTemporalType
+from ows.common.v20.decoders import kvp_decode_get_capabilities
+from ows.gml import Grid, IrregularAxis, RegularAxis, SpatioTemporalType
 from ows.swe import Field
-from ows.wcs import (
-    CoverageSummary, ServiceCapabilities, CoverageDescription
-)
-
-from ows.wcs.v20.decoders import kvp_decode_describe_coverage, kvp_decode_get_coverage
+from ows.wcs import CoverageDescription, CoverageSummary, ServiceCapabilities
 from ows.wcs.v20 import encoders as encoders_v20
+from ows.wcs.v20.decoders import (kvp_decode_describe_coverage,
+                                  kvp_decode_get_coverage)
 from ows.wcs.v21 import encoders as encoders_v21
 
-from datacube_ows.ogc_utils import resp_headers, get_service_base_url
-
 from datacube_ows.ogc_exceptions import WCS2Exception
-from datacube_ows.wcs2_utils import get_coverage_data
-
+from datacube_ows.ogc_utils import get_service_base_url, resp_headers
 from datacube_ows.ows_configuration import get_config
-
 from datacube_ows.utils import log_call
-
+from datacube_ows.wcs2_utils import get_coverage_data
 
 WCS_REQUESTS = ("DESCRIBECOVERAGE", "GETCOVERAGE")
 
@@ -78,7 +75,7 @@ def get_capabilities(args):
         include_coverage_summary = True
 
     capabilities = ServiceCapabilities.with_defaults_v20(
-        service_url=base_url+'/wcs',
+        service_url =base_url + '/wcs',
         allowed_operations=[
             'GetCapabilities', 'DescribeCoverage', 'GetCoverage'
         ],
@@ -90,17 +87,17 @@ def get_capabilities(args):
         access_constraints=[cfg.access_constraints],
         provider_name='',
         provider_site='',
-        individual_name=cfg.contact_info['person'],
-        organisation_name=cfg.contact_info['organisation'],
-        position_name=cfg.contact_info['position'],
-        phone_voice=cfg.contact_info['telephone'],
-        phone_facsimile=cfg.contact_info['fax'],
-        delivery_point=cfg.contact_info['address']['address'],
-        city=cfg.contact_info['address']['city'],
-        administrative_area=cfg.contact_info['address']['state'],
-        postal_code=cfg.contact_info['address']['postcode'],
-        country=cfg.contact_info['address']['country'],
-        electronic_mail_address=cfg.contact_info['email'],
+        individual_name=cfg.contact_info.person,
+        organisation_name=cfg.contact_info.organisation,
+        position_name=cfg.contact_info.position,
+        phone_voice=cfg.contact_info.telephone,
+        phone_facsimile=cfg.contact_info.fax,
+        delivery_point=cfg.contact_info.address.address,
+        city=cfg.contact_info.address.city,
+        administrative_area=cfg.contact_info.address.state,
+        postal_code=cfg.contact_info.address.postcode,
+        country=cfg.contact_info.address.country,
+        electronic_mail_address=cfg.contact_info.email,
         online_resource=base_url,
         # hours_of_service=,
         # contact_instructions=,
@@ -225,6 +222,7 @@ def create_coverage_description(cfg, product):
         coverage_subtype='RectifiedGridCoverage',
     )
 
+
 @log_call
 def desc_coverages(args):
     cfg = get_config()
@@ -269,8 +267,10 @@ def desc_coverages(args):
 
 
 import logging
+
 _LOG = logging.getLogger(__name__)
 
+
 @log_call
 def get_coverage(args):
     request_obj = kvp_decode_get_coverage(args)
